@@ -2,7 +2,7 @@
  *
  * This file is part of pyosmium. (https://osmcode.org/pyosmium/)
  *
- * Copyright (C) 2023 Sarah Hoffmann <lonvia@denofr.de> and others.
+ * Copyright (C) 2024 Sarah Hoffmann <lonvia@denofr.de> and others.
  * For a full list of authors see the git log.
  */
 #include <pybind11/pybind11.h>
@@ -15,6 +15,8 @@
 
 namespace py = pybind11;
 
+namespace {
+
 struct LastChangeHandler : public osmium::handler::Handler
 {
     osmium::Timestamp last_change;
@@ -27,6 +29,7 @@ struct LastChangeHandler : public osmium::handler::Handler
     }
 };
 
+} // namespace
 
 PYBIND11_MODULE(_replication, m)
 {
@@ -39,6 +42,5 @@ PYBIND11_MODULE(_replication, m)
             reader.close();
 
             return handler.last_change;
-        },
-        "Find the date of the most recent change in a file.");
+        });
 }
