@@ -2,9 +2,10 @@
 #
 # This file is part of pyosmium. (https://osmcode.org/pyosmium/)
 #
-# Copyright (C) 2023 Sarah Hoffmann <lonvia@denofr.de> and others.
+# Copyright (C) 2024 Sarah Hoffmann <lonvia@denofr.de> and others.
 # For a full list of authors see the git log.
-from typing import Sequence, Any, NamedTuple, Callable, Optional, Iterator, Iterable, TYPE_CHECKING, TypeVar, Generic, Tuple
+from typing import Sequence, Any, NamedTuple, Callable, Optional, Iterator, \
+                   Iterable, TYPE_CHECKING, TypeVar, Generic, Tuple, Union
 import datetime as dt
 
 import osmium.osm.mutable
@@ -66,7 +67,7 @@ class TagIterator:
 class TagList(Iterable[Tag]):
     """ A fixed list of tags. The list is exported as an unmutable,
         dictionary-like object where the keys are tag strings and the
-        items are :py:class:`osmium.osm.Tag`.
+        items are [Tags][osmium.osm.Tag].
     """
 
     def __init__(self, parent: 'cosm.TagContainerProtocol') -> None:
@@ -159,7 +160,7 @@ class NodeRef:
 
 class NodeRefList:
     """ A list of node references, implemented as
-        an immutable sequence of :py:class:`osmium.osm.NodeRef`. This class
+        an immutable sequence of [osmium.osm.NodeRef][]. This class
         is normally not used directly, use one of its subclasses instead.
     """
 
@@ -181,8 +182,8 @@ class NodeRefList:
     def ends_have_same_location(self) -> bool:
         """ True if the start and end node of the way are at the same location. "
             Expects that the coordinates of the way nodes have been loaded
-            (see :py:func:`osmium.SimpleHandler.apply_buffer` and
-            :py:func:`osmium.SimpleHandler.apply_file`).
+            ([SimpleHandler apply functions][osmium.SimpleHandler] and
+            [`FileProcessor.with_locations()`][osmium.FileProcessor.with_locations]).
             If the locations are not present then the function returns always true.
         """
         return self._list.ends_have_same_location(self._pyosmium_data)
@@ -213,18 +214,18 @@ class NodeRefList:
 
 class WayNodeList(NodeRefList):
     """ List of nodes in a way.
-        For its members see :py:class:`osmium.osm.NodeRefList`.
+        For its members see [`osmium.osm.NodeRefList`][].
     """
 
 
 class OuterRing(NodeRefList):
     """List of nodes in an outer ring.
-       For its members see :py:class:`osmium.osm.NodeRefList`.
+       For its members see [`osmium.osm.NodeRefList`][].
     """
 
 class InnerRing(NodeRefList):
     """ List of nodes in an inner ring. "
-        For its members see :py:class:`osmium.osm.NodeRefList`.
+        For its members see [`osmium.osm.NodeRefList`][].
     """
 
 class RelationMember:
@@ -271,7 +272,7 @@ class MemberIterator:
 
 class RelationMemberList:
     """ An immutable  sequence of relation members
-        ":py:class:`osmium.osm.RelationMember`.
+        [`osmium.osm.RelationMember`][].
     """
 
     def __init__(self, parent: 'cosm.COSMRelation') -> None:
@@ -303,8 +304,7 @@ class OSMObject(Generic[T_obj]):
         all common attributes.
     """
     _pyosmium_data: T_obj
-    tags: TagList
-    "(read-only) List of tags describing the object. See :py:class:`osmium.osm.TagList`."
+    _tags: TagList
 
     @property
     def id(self) -> int:
@@ -358,6 +358,13 @@ class OSMObject(Generic[T_obj]):
         """
         return self._pyosmium_data.user()
 
+    @property
+    def tags(self) -> TagList:
+        """ (read-only) List of tags describing the object.
+            See [`osmium.osm.TagList`][].
+        """
+        return self._tags
+
     def positive_id(self) -> int:
         """ Get the absolute value of the id of this object.
         """
@@ -370,6 +377,32 @@ class OSMObject(Generic[T_obj]):
         """
         return self._pyosmium_data.user_is_anonymous()
 
+    def type_str(self) -> str:
+        """ Return a single character identifying the type of the object.
+            The character is the same as used in OPL.
+        """
+        return 'o'
+
+    def is_node(self) -> bool:
+        """ Return true if the object is a Node object.
+        """
+        return isinstance(self, Node)
+
+    def is_way(self) -> bool:
+        """ Return true if the object is a Way object.
+        """
+        return isinstance(self, Way)
+
+    def is_relation(self) -> bool:
+        """ Return true if the object is a Relation object.
+        """
+        return isinstance(self, Relation)
+
+    def is_area(self) -> bool:
+        """ Return true if the object is a Way object.
+        """
+        return isinstance(self, Area)
+
 
 class Node(OSMObject['cosm.COSMNode']):
     """ Represents a single OSM node. It inherits all properties from OSMObjects
@@ -379,7 +412,7 @@ class Node(OSMObject['cosm.COSMNode']):
     def __init__(self, cnode: 'cosm.COSMNode'):
         self._pyosmium_data = cnode
         self._location: Optional['osmium.osm.Location'] = None
-        self.tags = TagList(self._pyosmium_data)
+        self._tags = TagList(self._pyosmium_data)
 
     def replace(self, **kwargs: Any) -> 'osmium.osm.mutable.Node':
         """ Create a mutable node replacing the properties given in the
@@ -396,13 +429,29 @@ class Node(OSMObject['cosm.COSMNode']):
     @property
     def location(self) -> 'osmium.osm.Location':
         """ The geographic coordinates of the node.
-            See :py:class:`osmium.osm.Location`.
+            See [`osmium.osm.Location`][].
         """
         if self._location is None:
             self._location = self._pyosmium_data.location()
 
         return self._location
 
+    @property
+    def lat(self) -> float:
+        """ Return latitude of the node.
+        """
+        return self.location.lat
+
+
+    @property
+    def lon(self) -> float:
+        """ Return longitude of the node.
+        """
+        return self.location.lon
+
+    def type_str(self) -> str:
+        return 'n'
+
     def __str__(self) -> str:
         if self._pyosmium_data.is_valid():
             return f'n{self.id:d}: location={self.location!s} tags={self.tags!s}'
@@ -422,7 +471,7 @@ class Way(OSMObject['cosm.COSMWay']):
 
     def __init__(self, cway: 'cosm.COSMWay'):
         self._pyosmium_data = cway
-        self.tags = TagList(self._pyosmium_data)
+        self._tags = TagList(self._pyosmium_data)
         self._nodes: Optional[WayNodeList] = None
 
     def replace(self, **kwargs: Any) -> 'osmium.osm.mutable.Way':
@@ -440,7 +489,7 @@ class Way(OSMObject['cosm.COSMWay']):
     @property
     def nodes(self) -> WayNodeList:
         """ (read-only) Ordered list of nodes.
-            See :py:class:`osmium.osm.WayNodeList`.
+            See [`osmium.osm.WayNodeList`][].
         """
         if self._nodes is None:
             self._nodes = WayNodeList(self._pyosmium_data, self._pyosmium_data.nodes())
@@ -461,12 +510,15 @@ class Way(OSMObject['cosm.COSMWay']):
     def ends_have_same_location(self) -> bool:
         """ True if the start and end node of the way are at the same location.
             Expects that the coordinates of the way nodes have been loaded
-            (see :py:func:`osmium.SimpleHandler.apply_buffer` and
-            :py:func:`osmium.SimpleHandler.apply_file`).
+            (see [SimpleHandler apply functions][osmium.SimpleHandler] and
+            [`FileProcessor.with_locations()`][osmium.FileProcessor.with_locations])
             If the locations are not present then the function returns always true.
         """
         return self._pyosmium_data.ends_have_same_location()
 
+    def type_str(self) -> str:
+        return 'w'
+
     def __str__(self) -> str:
         if self._pyosmium_data.is_valid():
             return f'w{self.id:d}: nodes={self.nodes!s} tags={self.tags!s}'
@@ -481,15 +533,12 @@ class Relation(OSMObject['cosm.COSMRelation']):
     """ Represents a OSM relation. It inherits the attributes from OSMObject
         and adds an ordered list of members.
     """
-    members: RelationMemberList
-    """(read-only) Ordered list of relation members.
-       See :py:class:`osmium.osm.RelationMemberList`.
-    """
+    _members: RelationMemberList
 
     def __init__(self, crelation: 'cosm.COSMRelation'):
         self._pyosmium_data = crelation
-        self.tags = TagList(self._pyosmium_data)
-        self.members = RelationMemberList(self._pyosmium_data)
+        self._tags = TagList(self._pyosmium_data)
+        self._members = RelationMemberList(self._pyosmium_data)
 
     def replace(self, **kwargs: Any) -> 'osmium.osm.mutable.Relation':
         """ Create a mutable relation replacing the properties given in the
@@ -503,6 +552,16 @@ class Relation(OSMObject['cosm.COSMRelation']):
         """
         return osmium.osm.mutable.Relation(self, **kwargs)
 
+    @property
+    def members(self) -> RelationMemberList:
+        """(read-only) Ordered list of relation members.
+           See [`osmium.osm.RelationMemberList`][].
+        """
+        return self._members
+
+    def type_str(self) -> str:
+        return 'r'
+
     def __str__(self) -> str:
         if self._pyosmium_data.is_valid():
             return f"r{self.id:d}: members={self.members!s}, tags={self.tags!s}"
@@ -555,7 +614,7 @@ class Area(OSMObject['cosm.COSMArea']):
 
     def __init__(self, carea: 'cosm.COSMArea') -> None:
         self._pyosmium_data = carea
-        self.tags = TagList(self._pyosmium_data)
+        self._tags = TagList(self._pyosmium_data)
 
     def from_way(self) -> bool:
         """ Return true if the area was created from a way, false if it was
@@ -593,6 +652,9 @@ class Area(OSMObject['cosm.COSMArea']):
         """
         return InnerRingIterator(self._pyosmium_data, oring)
 
+    def type_str(self) -> str:
+        return 'a'
+
     def __str__(self) -> str:
         if self._pyosmium_data.is_valid():
             return f"a{self.id:d}: num_rings={self.num_rings()}, tags={self.tags!s}"
@@ -609,12 +671,12 @@ class Changeset:
     """
     _pyosmium_data: 'cosm.COSMChangeset'
     _bounds: Optional['osmium.osm.Box']
-    tags: TagList
+    _tags: TagList
 
     def __init__(self, carea: 'cosm.COSMChangeset') -> None:
         self._pyosmium_data = carea
         self._bounds = None
-        self.tags = TagList(self._pyosmium_data)
+        self._tags = TagList(self._pyosmium_data)
 
     @property
     def id(self) -> int:
@@ -670,6 +732,13 @@ class Changeset:
 
         return self._bounds
 
+    @property
+    def tags(self) -> TagList:
+        """ (read-only) List of tags describing the object.
+            See [`osmium.osm.TagList`][].
+        """
+        return self._tags
+
     def user_is_anonymous(self) -> bool:
         """ Check if the user anonymous. If true, the uid does not uniquely
             identify a single user but only the group of all anonymous users
@@ -677,6 +746,9 @@ class Changeset:
         """
         return self._pyosmium_data.user_is_anonymous()
 
+    def type_str(self) -> str:
+        return 'c'
+
     def __str__(self) -> str:
         if self._pyosmium_data.is_valid():
             return f'c{self.id:d}: closed_at={self.closed_at!s}, bounds={self.bounds!s}, tags={self.tags!s}'
@@ -688,6 +760,4 @@ class Changeset:
                                         'tags')
 
 
-
-
-
+OSMEntity = Union[Node, Way, Relation, Area, Changeset]
