@@ -7,12 +7,43 @@ import pytest
 
 import osmium as o
 
+from helpers import CountingHandler
+
+class NullHandler:
+
+    def node(self, n):
+        pass
+
 def _run_file(fn):
-    rd = o.io.Reader(fn)
-    try:
-        o.apply(rd, o.SimpleHandler())
-    finally:
-        rd.close()
+    with o.io.Reader(fn) as rd:
+        o.apply(rd, NullHandler())
+
+
+@pytest.mark.parametrize('as_string', [True, False])
+def test_file_simple(tmp_path, as_string):
+    fn = tmp_path / 'text.opl'
+    fn.write_text('n1')
+
+    if as_string:
+        fn = str(fn)
+
+    for n in o.FileProcessor(o.io.File(fn)):
+        assert n.is_node()
+        assert n.id == 1
+
+
+@pytest.mark.parametrize('as_string', [True, False])
+def test_file_with_format(tmp_path, as_string):
+    fn = tmp_path / 'text.txt'
+    fn.write_text('n1')
+
+    if as_string:
+        fn = str(fn)
+
+    for n in o.FileProcessor(o.io.File(fn, 'opl')):
+        assert n.is_node()
+        assert n.id == 1
+
 
 def test_node_only(test_data):
     _run_file(test_data('n1'))
@@ -40,15 +71,14 @@ def test_relation_with_tags(test_data):
 
 def test_broken_timestamp(test_data):
     fn = test_data('n1 tx')
-    try:
-        rd = o.io.Reader(fn)
+
+    with o.io.Reader(fn) as rd:
         with pytest.raises(RuntimeError):
-            o.apply(rd, o.SimpleHandler())
-    finally:
-        rd.close()
+            o.apply(rd, NullHandler())
 
 
-def test_file_header(tmp_path):
+@pytest.mark.parametrize('as_string', [True, False])
+def test_file_header(tmp_path, as_string):
     fn = tmp_path / 'empty.xml'
     fn.write_text("""<?xml version='1.0' encoding='UTF-8'?>
     <osm version="0.6" generator="test-pyosmium" timestamp="2014-08-26T20:22:02Z">
@@ -56,11 +86,20 @@ def test_file_header(tmp_path):
     </osm>
     """)
 
-    rd = o.io.Reader(str(fn))
-    try:
+    if as_string:
+        fn = str(fn)
+
+    with o.io.Reader(fn) as rd:
         h = rd.header()
         assert not h.has_multiple_object_versions
         assert h.box().valid()
         assert h.box().size() == 64800.0
-    finally:
-        rd.close()
+
+
+def test_reader_with_filebuffer():
+    rd = o.io.Reader(o.io.FileBuffer('n1 x4 y1'.encode('utf-8'), 'opl'))
+    handler = CountingHandler()
+
+    o.apply(rd, handler)
+
+    assert handler.counts == [1, 0, 0, 0]
