@@ -2,7 +2,7 @@
 #
 # This file is part of Pyosmium.
 #
-# Copyright (C) 2022 Sarah Hoffmann.
+# Copyright (C) 2024 Sarah Hoffmann.
 import re
 from itertools import count
 import pytest
@@ -71,8 +71,8 @@ def test_invalid_location():
     with pytest.raises(o.InvalidLocationError):
         lon = loc.lon
     # these two don't raise an exception
-    lat = loc.lat_without_check()
-    lon = loc.lon_without_check()
+    assert loc.lat_without_check() is not None
+    assert loc.lon_without_check() is not None
 
 
 def test_valid_location():
@@ -85,8 +85,22 @@ def test_valid_location():
     assert repr(loc) == 'osmium.osm.Location(x=-10000000, y=100000000)'
 
 
-def test_node_attributes(test_importer):
-    def node(n):
+@pytest.mark.parametrize('attrname', ['id', 'deleted', 'visible',
+                                      'changeset', 'uid', 'timestamp', 'user',
+                                      'tags'])
+@pytest.mark.parametrize('osmdata', ['n1 v5 c58674 uänonymous',
+                                     'w34 Nn34',
+                                     'r45 Tfoo=rte'])
+def test_object_attribute_do_not_overwrite(opl_buffer, attrname, osmdata):
+    for n in o.FileProcessor(opl_buffer(osmdata)):
+        with pytest.raises(AttributeError):
+            setattr(n, attrname, 3)
+
+
+def test_node_attributes(opl_buffer):
+    node_data = 'n1 v5 c58674 t2014-01-31T06:23:35Z i42 uänonymous'
+
+    for n in o.FileProcessor(opl_buffer(node_data)):
         assert n.deleted == False
         assert n.visible == True
         assert n.version == 5
@@ -96,23 +110,30 @@ def test_node_attributes(test_importer):
         assert n.timestamp == mkdate(2014, 1, 31, 6, 23, 35)
         assert n.user == u'änonymous'
         assert n.positive_id() == 1
+        assert n.is_node()
+        assert not n.is_way()
+        assert n.type_str() == 'n'
         assert str(n) == 'n1: location=invalid tags={}'
         assert repr(n) == "osmium.osm.Node(id=1, deleted=False, visible=True, version=5, changeset=58674, uid=42, timestamp=datetime.datetime(2014, 1, 31, 6, 23, 35, tzinfo=datetime.timezone.utc), user='änonymous', tags=osmium.osm.TagList({}), location=osmium.osm.Location())"
+        break
+    else:
+        assert False
 
-    assert 1 == test_importer('n1 v5 c58674 t2014-01-31T06:23:35Z i42 uänonymous',
-                              node=node)
 
+def test_node_location(opl_buffer):
+    for n in o.FileProcessor(opl_buffer("n1 x4 y5")):
+        assert n.lat == 5.0
+        assert n.lon == 4.0
 
 
 @pytest.mark.parametrize('nid', (23, 0, -68373, 17179869418, -17179869417))
-def test_node_positive_id(test_importer, nid):
-
-    def node(n):
+def test_node_positive_id(opl_buffer, nid):
+    for n in o.FileProcessor(opl_buffer(f"n{nid} v5 c58674")):
         assert n.id == nid
         assert n.positive_id() == abs(nid)
-
-    assert 1 == test_importer('n{} v5 c58674'.format(nid),
-                              node=node)
+        break
+    else:
+        assert False
 
 
 def test_way_attributes(test_importer):
@@ -127,6 +148,8 @@ def test_way_attributes(test_importer):
         assert o.timestamp == mkdate(2014, 1, 31, 6, 23, 35)
         assert o.user == 'anonymous'
         assert o.positive_id() == 1
+        assert o.is_way()
+        assert o.type_str() == 'w'
         assert not o.is_closed()
         assert not o.ends_have_same_id()
         assert not o.ends_have_same_location()
@@ -142,6 +165,15 @@ def test_way_attributes(test_importer):
                               way=way, locations=True)
 
 
+def test_way_attribute_do_not_overwrite(opl_buffer):
+    data = """\
+           w34 Nn34
+           """
+    for w in o.FileProcessor(opl_buffer(data)):
+        with pytest.raises(AttributeError):
+            w.nodes = [3,4,5]
+
+
 def test_relation_attributes(test_importer):
     def relation(o):
         assert o.id == 1
@@ -154,6 +186,8 @@ def test_relation_attributes(test_importer):
         assert o.timestamp == mkdate(2014, 1, 31, 6, 23, 35)
         assert o.user == ' anonymous'
         assert o.positive_id() == 1
+        assert o.is_relation()
+        assert o.type_str() == 'r'
 
         assert str(o) == 'r1: members=[w1], tags={}'
         assert repr(o) == "osmium.osm.Relation(id=1, deleted=False, visible=True, version=5, changeset=58674, uid=42, timestamp=datetime.datetime(2014, 1, 31, 6, 23, 35, tzinfo=datetime.timezone.utc), user=' anonymous', tags=osmium.osm.TagList({}), members=osmium.osm.RelationMemberList([osmium.osm.RelationMember(ref=1, type='w', role='')]))"
@@ -165,6 +199,15 @@ def test_relation_attributes(test_importer):
                               relation=relation)
 
 
+def test_relation_attribute_do_not_overwrite(opl_buffer):
+    data = """\
+           r34 Mn23@,w34@
+           """
+    for r in o.FileProcessor(opl_buffer(data)):
+        with pytest.raises(AttributeError):
+            r.members = [3,4,5]
+
+
 def test_area_from_way_attributes(area_importer):
     def area(o):
         assert o.id == 46
@@ -179,6 +222,8 @@ def test_area_from_way_attributes(area_importer):
         assert o.positive_id() == 46
         assert o.orig_id() == 23
         assert o.from_way() == True
+        assert o.is_area()
+        assert o.type_str() == 'a'
         assert o.is_multipolygon() == False
         assert o.num_rings() == (1, 0)
         assert len(list(o.outer_rings())) == 1
@@ -243,9 +288,20 @@ def test_changest_attributes(area_importer):
         assert 515288620 == c.bounds.top_right.y
         assert -1465242 == c.bounds.bottom_left.x
         assert 515288506 == c.bounds.bottom_left.y
+        assert c.type_str() == 'c'
         assert str(c) == 'c34: closed_at=2005-04-09 20:54:39+00:00, bounds=(-0.1465242/51.5288506 -0.1464925/51.5288620), tags={}'
         assert repr(c) == "osmium.osm.Changeset(id=34, uid=1, created_at=datetime.datetime(2005, 4, 9, 19, 54, 13, tzinfo=datetime.timezone.utc), closed_at=datetime.datetime(2005, 4, 9, 20, 54, 39, tzinfo=datetime.timezone.utc), open=False, num_changes=2, bounds=osmium.osm.Box(bottom_left=osmium.osm.Location(x=-1465242, y=515288506), top_right=osmium.osm.Location(x=-1464925, y=515288620)), user='Steve', tags=osmium.osm.TagList({}))"
 
     assert 1 == area_importer('c34 k2 s2005-04-09T19:54:13Z e2005-04-09T20:54:39Z '
                                'd34 i1 uSteve x-0.1465242 y51.5288506 X-0.1464925 Y51.5288620',
                               changeset=changeset)
+
+
+def test_entity_operations():
+    assert not o.osm.NOTHING
+    assert o.osm.NODE
+
+    assert o.osm.AREA | o.osm.NODE | o.osm.WAY | o.osm.RELATION == o.osm.OBJECT
+    assert o.osm.ALL & o.osm.RELATION == o.osm.RELATION
+
+    assert ~o.osm.CHANGESET == o.osm.OBJECT
